#!/bin/bash

# clunkgal: static photo gallery generation tool
# Copyright (c) Ashley Hooper 2007 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 


INDEXFILE=index.shtml

BASEDIR=`pwd`
#BASEDIR=~/scripts/web/clunkgal
SITEROOT="/var/www/clunker.dyndns.org"
SUBDIR='photos'
ALL_EXTENSIONS='jpg JPG png PNG'
PIC_EXTENSIONS='jpg JPG png PNG'
# There is scope to add other types of files in the future, but for now we are
# only interested in picture files.

# Process the user-specified subdirectory (if any), otherwise the whole tree
if [ -n $1 ]; then
  SUBDIR='photos/'$1
else
  SUBDIR='photos' 
fi

GALLERY="$SITEROOT/$SUBDIR"
IMGDIR="/images"
SUBDIRSPERROW=5
THUMBSPERROW=4

# See the ImageMagick convert command syntax for explanation of the below
THUMBSIZE=150x100
MANIPSIZE=x550
# Width at which a manipulated image becomes panoramic
PANO_THRESHOLD=900

# Copy folder and navigation button images

mkdir -p $SITEROOT/$IMGDIR
cp $BASEDIR/templ_icon_folder.png  $SITEROOT/$IMGDIR
cp $BASEDIR/templ_icon_up.png      $SITEROOT/$IMGDIR
cp $BASEDIR/templ_icon_back.png    $SITEROOT/$IMGDIR
cp $BASEDIR/templ_icon_forward.png $SITEROOT/$IMGDIR

parentpic="$IMGDIR/templ_icon_up.png"


# Build the list of directories to recurse through

for rawdir in `find """$GALLERY""" -name '.manip' -o -name '.thumb' -o -name '.html' -prune -o -type d -print | sort | tr ' ' '@'`
do
  dir=`echo $rawdir | sed 's/@/ /g'`
  topdir=`echo $dir | sed 's/.*\///'`
  COUNT=0
  cd "$dir"
  (
    echo '<!--#set var="pagetitle" value="the haberdashery: photos: '$topdir'" -->'
    cat $BASEDIR/templ_dir.shtml
    echo '<div class="gallery-content">' 
    echo '<table cellpadding="4"><div id="subdirs"><tr>'
    echo '<td>'
    echo '<a href=".."><img src='$parentpic'><br><b>Go up</b></a>'
    echo '</td>'
  ) > $INDEXFILE

  for templatedir in .manip .thumb .html
  do
    if [ -d $templatedir ]; then
      rm -rf $templatedir
    fi
  done


# Now enumerate directories below this one to display subdir hyperlinks

  for rawsubdir in `find -maxdepth 1 -name '.' -o -name '.manip' -o -name '.thumb' -o -name '.html' -prune -o -type d -print | sort | tr ' ' '@'`
  do
    subdir=`echo $rawsubdir | sed 's/@/ /g'`
    nicesubdir=`echo $subdir | sed 's/.\///'`
    let COUNT=COUNT+1
    let SPLIT=COUNT%SUBDIRSPERROW
    if [ "$SPLIT" -eq "0" ]; then
      echo '</tr><tr>' >> $INDEXFILE
    fi
    (
      echo '<td>'
      folderpic="$IMGDIR/templ_icon_folder.png"
      echo '<a href="'$nicesubdir'"><img src="'$folderpic'"><br>'$nicesubdir'</a>'
      echo '</td>'
    ) >> $INDEXFILE
  done
  echo '</tr></div></table><hr>' >> $INDEXFILE

  COUNT=0
  (
    echo '<table>'
    echo '<div id="photos"><tr>'
    echo '<td bgcolor="black" width="150">'
    echo 'In folder:<br><i><b>'$topdir'</b></i>'
    echo '</td>'
  ) >> $INDEXFILE
  mkdir .thumb  .manip  .html


  # Now enumerate all files, and select files with valid extensions as per
  # $ALL_EXTENSIONS variable.

  TOTAL=0
  for file in *.*;
  do
    for extension in $ALL_EXTENSIONS
    do 
      if [ $extension == ${file##*.} ]; then
        allitems[$TOTAL]="$file"
#        echo $file - allitems[$TOTAL] = ${allitems[$TOTAL]}
        let TOTAL+=1
      fi
    done
  done


  # Now walk the array of 'interesting' files and process them

  for (( i = 0 ; i < TOTAL ; i++ )); do
    item=${allitems[i]}
    # echo $item

    LAST=$COUNT
    let COUNT+=1
    let NEXT=COUNT+1

    if [ $LAST -lt 1 ]; then
      LAST=".."
    else
      LAST="$LAST.shtml"
    fi
    if [ $NEXT -gt $TOTAL ]; then
      NEXT=".."
    else
      NEXT="$NEXT.shtml"
    fi

    itembasename=${item%%.*}
    itemhtmlfile=".html/$COUNT.shtml"
    echo "Processing '$itembasename' ($COUNT of $TOTAL in folder '$topdir')"
    for pic_extension in $PIC_EXTENSIONS
    do
      if [ $pic_extension == ${item##*.} ]; then
        convert -resize $THUMBSIZE "$item" .thumb/thumb_$COUNT
        convert -resize $MANIPSIZE "$item" .manip/manip_$COUNT

        imagexsize=`identify ".manip/manip_$COUNT" | sed 's/.* \([0-9]*\)x[0-9]* .*/\1/'`

        let SPLIT=COUNT%THUMBSPERROW
        if [ "$SPLIT" -eq "0" ]; then
          echo '</tr><tr>' >> $INDEXFILE
        fi
  
        # Add a thumbnail to the directory index
        (
          echo '<td>'
          echo '<a href=".html/'$COUNT'.shtml" title="'$itembasename'"><img src=".thumb/thumb_'$COUNT'" alt="'$itembasename'"></a>'
          echo '</td>'
        ) >> $INDEXFILE


        # Generate the html file with the manipulated image
        (
          echo '<!--#set var="pagetitle" value="the haberdashery: photos: '$topdir'" -->'
          cat $BASEDIR/templ_photo.shtml
          echo '<div class="gallery-content">' 
          echo '<nobr><b><a href="'$LAST'"><img src="'$IMGDIR'/templ_icon_back.png"></a> <a href=".."><img src="'$IMGDIR'/templ_icon_up.png"></a> <a href="'$NEXT'"><img src="'$IMGDIR'/templ_icon_forward.png"></a> | Title: </b>'$itembasename' '
          if [ $imagexsize -gt $PANO_THRESHOLD ]; then
            echo '<b><font color="yellow">-&nbsp;panorama&nbsp;-></font></b>'
          fi
          echo '<a href="../'$item'">[view original]</a></nobr>'
          echo '<a href="'$NEXT'"><img src="../.manip/manip_'$COUNT'" border=0></a>'
          echo '<br><font size=-1>Generated using <a href="http://clunker.dyndns.org/tech/clunkgal/">Clunkgal</a> static photo gallery software</font>'
          if [ -f "$itembasename.cap" ]; then
            echo '<pre>'
            cat "$itembasename.cap"
            echo '</pre>'
          fi
        ) > "$itemhtmlfile"
      fi
    done
  done

  echo '</tr></table>' >> $INDEXFILE

done
